#!/usr/bin/env python

SUITES_FILE = "suites.data"
SQL_DATA = "files-per-language.data"
OUTPUT_FILE = "size-file-per-language.csv"
STATS_FILE = "stats.data"
FIG = "charts/size-file-per-language.png"

LANGUAGES = ["ansic", "cpp", "java", "xml", "sh", "python", "perl", "lisp",
             "asm", "fortran", "cs", "php"]

# from David Wheeler SLOCCOUNT's /usr/bin/break_filelist
MAPPING = {
    'ansic': ('c', 'ec', 'ecp', 'pgc', 'h'),
    'cpp': ('C', 'cpp', 'cxx', 'cc', 'H', 'hpp', 'hh'),
    'java': ('java',),
    'xml': ('xml',),
    'sh': ('sh', 'bash'),
    'python': ('py',),
    'perl': ('pl', 'pm', 'perl', 'ph'),
    'lisp': ('lisp', 'el', 'scm', 'sc'),
    'asm': ('s', 'S', 'asm'),
    'fortran': ('f', 'F', 'f77', 'F77'),
    'cs': ('cs',),
    'php': ('php', 'php3', 'php4', 'php5', 'php6')
    }

N_PLOT = 6

FORMATS = ("*-k", "o-k", "v-k", "^-k", "s-k", ".-k")
assert len(FORMATS) >= N_PLOT

import sys
sys.path.append('../debsources/python')

import csv
from debsources_data import correct_suite_name, split_suite
from app.extract_stats import extract_stats
from pprint import pprint
import matplotlib.pyplot as plt

def get_data():
    return extract_stats(filename=STATS_FILE)

def get_suites():
    suites = []
    with open(SUITES_FILE) as f:
        for line in f:
            line = line.strip()
            if line[:3] != "sid":
                suites.append(line)
    return suites

def plot(plots, suites):
    plt.subplots_adjust(bottom=0.2)
    
    for lang, form in zip(LANGUAGES[:N_PLOT], FORMATS[:N_PLOT]):
        plt.plot([s["date"] for s in suites], plots[lang], form)
    
    plt.legend(LANGUAGES[:N_PLOT], loc=2)
    plt.ylabel("SLOC")
    
    for suite in suites:
        plt.annotate(correct_suite_name(suite["name"]),
                     (suite["date"], 0), # aligned on the x-axis
                     ha="center", # horizontal alignment
                     rotation=90,
                     color="black",
                     textcoords="offset points",
                     xytext=(0, -30),
                     )
    plt.savefig(FIG)

def compute(sql_data, output_file, suites):
    data = dict()
    
    with open(sql_data) as f:
        for line in f:
            if line.strip() == "":
                continue
            values = line.split('|')
            values = [v.strip() for v in values]
            ext, suite, files = values
            data[suite + "_" + ext] = int(files)
    
    stats = get_data()
    
    plots = dict()
    for lang in LANGUAGES:
        plots[lang] = []
    
    with open(output_file, "w") as f:
        w = csv.writer(f, quoting=csv.QUOTE_ALL)
        w.writerow(["suite"] + LANGUAGES)
        
        for suite in suites:
            name = suite["name"]
            row = [name]
            
            for lang in LANGUAGES:
                sloc = int(stats["debian_" + name + ".sloccount." + lang])
                files = 0
                for ext in MAPPING[lang]:
                    try:
                        files += data[name + "_" + ext]
                    except: # extension not found in our data
                        pass
                try:
                    row.append(sloc / files)
                    plots[lang].append(sloc / files)
                except: # 0 sloc, 0 file
                    row.append("-")
                    plots[lang].append(None)
            w.writerow(row)
    
    plot(plots, suites)

if __name__ == "__main__":
    suites = get_suites()
    suites = [split_suite(suite) for suite in suites]
    
    compute(SQL_DATA, OUTPUT_FILE, suites)
