#!/usr/bin/python

STATS_FILE = "stats.data"
SUITES_FILE = "suites.data"
FIGS_DIR = "./charts"

import sys
sys.path.append('../debsources/python')

from app.extract_stats import extract_stats
from debsources_data import split_suite, correct_suite_name

import os
import matplotlib.pyplot as plt
from matplotlib.dates import date2num
from matplotlib.ticker import ScalarFormatter
from mpl_toolkits.axes_grid1 import host_subplot
import mpl_toolkits.axisartist as AA

import datetime

from pprint import pprint

def get_data():
    return extract_stats(filename=STATS_FILE)

def get_suites():
    suites = []
    with open(SUITES_FILE) as f:
        for line in f:
            line = line.strip()
            if line[:3] != "sid":
                suites.append(line)
    return suites

def format_scientific_axis(obj):
    formatter = ScalarFormatter()
    formatter.set_scientific(True)
    formatter.set_powerlimits((-3,3))
    obj.yaxis.set_major_formatter(formatter)

def unique_histogram(x, arr_y, debian_labels):
    def setup(suppl_axes):
        host = host_subplot(111, axes_class=AA.Axes)
        plt.tight_layout()
        plt.subplots_adjust(left=0.2, right=0.7, bottom=0.2, top=0.9)
        
        par1 = host.twinx()
        pars = []
        for _ in range(suppl_axes):
            pars.append(host.twinx())
        # par1 is the "standard" right axis, pars are the others
        return (host, plt, par1, pars)

    def add_axis(obj, offset, position="right"):
        new_fixed_axis = obj.get_grid_helper().new_fixed_axis
        obj.axis["right"] = new_fixed_axis(loc=position,
                                            axes=obj,
                                            offset=(offset, 0))
        obj.axis["right"].toggle(all=True)
    
    def set_ylabels(axis, label):
        axis.set_ylabel(label)
    
    def set_xlabels(obj, debian_labels):
        for label in debian_labels:
            obj.annotate(correct_suite_name(label[0]),
                         (label[1][0], 0), # aligned on the x-axis
                         ha="center", # horizontal alignment
                         rotation=90,
                         color="black",
                         textcoords="offset points",
                         xytext=(0, -30),
                         )
    
    def plot(obj, x, y, form, label):
        p = obj.plot(x, y, form, label=label)
        return p
    
    def compute():
        # y axes
        labels_axes = ("du (GB)", "packages (k)", "files (M)", "sloc (M)",
                       "ctags (M)")
        
        # line types
        formats = ("*-k", "o-k", "v-k", "^-k", "s-k")
        
        assert len(arr_y) == len(labels_axes) == len(formats)
        
        # we get a plot, an host, the right axis and the other axes
        (host, plt, par1, pars) = setup(len(arr_y)-2)
        
        # we set up the other axes
        add_axis(pars[0], 60, position="right")
        add_axis(pars[1], 120, position="right")
        add_axis(pars[2], -60, position="left")
        
        # x-axis label
        #host.set_xlabel("releases")
        
        # y-axes labels
        set_ylabels(pars[2], labels_axes[0])
        set_ylabels(host, labels_axes[1])
        set_ylabels(par1, labels_axes[2])
        set_ylabels(pars[0], labels_axes[3])
        set_ylabels(pars[1], labels_axes[4])
        
        # we add the "release names" labels
        set_xlabels(host, debian_labels)
        
        # scientific format for the y-axes
        [format_scientific_axis(obj) for obj in [host, par1] + pars]
        
        # and we can plot our data
        [plot(obj, x, y, form, lab) for (obj, y, form, lab)
         in zip([pars[2], host, par1, pars[0], pars[1]],
                arr_y, formats, labels_axes)]
        
        host.legend(loc=2)
        
        plt.grid(True)
        plt.savefig(os.path.join(FIGS_DIR, "size-combined.png"))
        plt.close()
    
    compute()


    # TODO: power of 10 at the top of the axes??
    
    # NOTE:
    # dt = datetime.datetime.fromordinal(ix)
    # ValueError: ordinal must be >= 1
    # solution -> number of axes must be equal to number of plots (yeah...)


def histograms():
    suites = get_suites()
    data = get_data()
    
    def compute(metric, yunit="", divide_y=1):
        plt.subplots_adjust(bottom=0.2)
        
        plotx = []
        ploty = []
        labels = []
        for suite in suites:
            suite = split_suite(suite)
            # we extract correct data
            x = suite["date"]
            y = float(data["debian_" + suite["name"] + "." + metric]) / divide_y
            # add it to the plot
            plotx.append(x)
            ploty.append(y)
            # annotations (label for release name)
            labels.append((suite["name"], (x, y)))
        
            plt.annotate(correct_suite_name(suite["name"]), (x, 0),
                         ha="center", # horizontal alignment
                         rotation=90,
                         color="black",
                         textcoords="offset points",
                         xytext=(8, -30),
                         )

        
        # axis labels
        #plt.xlabel("releases")
        if yunit != "":
            ylabel = metric.replace("_", " ") + " (" + yunit + ")"
        else:
            ylabel = metric
        plt.ylabel(ylabel)
        
        # and we plot
        plt.bar(plotx, ploty, width=200.0, color="black")
        plt.savefig(os.path.join(FIGS_DIR, "size-" + metric + ".png"))
        plt.close()
        
        return (plotx, ploty, labels)
    
    (x, y1, debian_labels) = compute("disk_usage", yunit="GB", divide_y=10**6)
    (_, y2, _) = compute("source_packages", yunit="k", divide_y=10**3)
    (_, y3, _) = compute("source_files", yunit="M", divide_y=10**6)
    (_, y4, _) = compute("sloccount", yunit="M", divide_y=10**6)
    (_, y5, _) = compute("ctags", yunit="M", divide_y=10**6)
    
    return (x, [y1, y2, y3, y4, y5], debian_labels)

def main():
    (x, ys, labs) = histograms()
    # we need to store this, it's used in the other histogram
    unique_histogram(x, ys, labs)

if __name__ == "__main__":
    main()
