#!/usr/bin/env python

SUITES_FILE = "suites.data"
FIGS_DIR = "./charts"
RELEASE_DUR_FILE = "release-duration.data"
STATS_FILE = "stats.data"
CSV_FILE = "sloc-evolution.csv"

##################################################
LANGUAGES_ALL = ["ansic", "cpp", "java", "xml", "sh", "python", "perl", "lisp",
                 "asm", "fortran", "cs", "php", "makefile", "pascal", "ruby",
                 "sql", "ml", "tcl", "objc", "haskell", "ada", "erlang", "yacc",
                 "f90", "exp", "lex", "awk", "jsp", "csh", "vhdl", "sed",
                 "modula3", "cobol"]
HATCHS = ["/", "o", "+", "O", ".", "\\", "*"]
FORMATS = ["*-k", "o-k", "v-k", "^-k", "s-k", "x-k"]

GRAPH_LANGS = 6 # number of languages used for the graph
CSV_LANGS = 10  # number of languages used for the csv table

assert GRAPH_LANGS <= len(LANGUAGES_ALL)
LANGUAGES_GRAPH = LANGUAGES_ALL[:GRAPH_LANGS]

assert CSV_LANGS <= len(LANGUAGES_ALL)
LANGUAGES_CSV = LANGUAGES_ALL[:CSV_LANGS]

assert GRAPH_LANGS <= len(HATCHS)
HATCHS = HATCHS[:GRAPH_LANGS]

assert GRAPH_LANGS <= len(FORMATS)
FORMATS = FORMATS[:GRAPH_LANGS]
##################################################

import sys
sys.path.append('../debsources/python')

from app.extract_stats import extract_stats
from debsources_data import split_suite, correct_suite_name

import os
import csv
from datetime import datetime
import matplotlib.pyplot as plt
from matplotlib.ticker import FuncFormatter

from pprint import pprint

def get_data():
    return extract_stats(filename=STATS_FILE)

def get_suites():
    suites = []
    with open(SUITES_FILE) as f:
        for line in f:
            line = line.strip()
            if line[:3] != "sid":
                suites.append(line)
    return suites

def output_csv(data, suites):
    def value_output(value, total):
        return "%.1f (%.1f\\%%)" % (value / float(10**6),
                                  (value / float(total)) * 100.)
    
    with open(CSV_FILE, "w") as f:
        w = csv.writer(f, quoting=csv.QUOTE_ALL)
        w.writerow(["release"] + LANGUAGES_CSV)
        for suite in suites:
            name = suite["name"]
            line = [value_output(data["debian_" + name + ".sloccount." + lang],
                                 data["debian_" + name + ".sloccount"])
                    for lang in LANGUAGES_CSV]
            w.writerow([name] + line)

def sloc_percentage(data, suites):
    plt.subplots_adjust(bottom=0.2)
    
    plotx = []
    ploty = dict()
    ploty = dict()
    
    for lang in LANGUAGES_GRAPH:
        ploty[lang] = []
    # we need relative SLOCcounts
    for suite in suites:
        name = suite["name"]
        
        # labels
        plt.annotate(correct_suite_name(name), (suite["date"], 0),
                     ha="center", # horizontal alignment
                     rotation=90,
                     color="black",
                     textcoords="offset points",
                     xytext=(8, -25),
                     )
        
        plotx.append(suite["date"])
        
        sloc_total = float(data["debian_" + name + ".sloccount"])
        
        for lang in LANGUAGES_GRAPH:
            ploty[lang].append((data["debian_" + name
                                     + ".sloccount." + lang] / sloc_total))
    
    p = []
    for (lang, form) in zip(LANGUAGES_GRAPH, FORMATS):
        p.append(plt.plot(plotx, ploty[lang], form))
    
    plt.legend([z[0] for z in p], LANGUAGES_GRAPH, loc=2)
    
    # percent on the y-axis
    plt.gca().yaxis.set_major_formatter(FuncFormatter(
            lambda x, y: str(int(x * 100)) + "%"))
    plt.ylabel("SLOC")

    plt.savefig(os.path.join(FIGS_DIR, "sloc-evol-rel.png"))
    

def sloc_hist(data, suites):
    plt.subplots_adjust(bottom=0.2)
    
    plotx = []
    ploty = dict()
    
    for lang in LANGUAGES_GRAPH:
        ploty[lang] = []
    for suite in suites:
        # labels
        plt.annotate(correct_suite_name(suite["name"]), (suite["date"], 0),
                     ha="center", # horizontal alignment
                     rotation=90,
                     color="black",
                     textcoords="offset points",
                     xytext=(8, -25),
                     )
        
        # data
        plotx.append(suite["date"])
        for lang in LANGUAGES_GRAPH:
            ploty[lang].append(
                (data["debian_" + suite["name"] + ".sloccount." + lang])
                / float(10**6))
    
    # bottom is used to compute the cumulative values
    bottom = [[0 for _ in range(len(suites))]]
    
    for i in range(len(LANGUAGES_GRAPH)-1):
        bottom.append([x+y for x,y in zip(bottom[i],
                                          ploty[LANGUAGES_GRAPH[i]])])
    
    p = []
    
    for i in range(len(LANGUAGES_GRAPH)):
        lang = LANGUAGES_GRAPH[i]
        hatch = HATCHS[i]
        p.append(plt.bar(plotx, ploty[lang],
                         width=250.0,
                         color="white",
                         edgecolor='black',
                         hatch=hatch,
                         bottom=bottom[i]))
    
    plt.legend(tuple(p), LANGUAGES_GRAPH, loc=2)
    plt.ylabel("SLOC (M)")
    
    plt.savefig(os.path.join(FIGS_DIR, "sloc-evol-abs.png"))
    plt.close()

def main():
    data = get_data()
    suites = get_suites()
    suites = [split_suite(suite) for suite in suites]
    
    sloc_hist(data, suites)
    sloc_percentage(data, suites)
    
    output_csv(data, suites)

if __name__ == "__main__":
    main()
